function [Nb,Nnb,Ab,Anb,Dfb,Dfnb]=DetectBreakersEnsemble(FileName, Fs, NumGauges, plots)
%% Main function to perform detection of breaking waves in surface elevation fluctuations time series data.
% Basic processing parameters must be set manually, in the first cell (Basic Settings).
% For algorithm details, recommended measuremnts and processing parameters see:
% Liberzon et al. (2019)
% Detection of breaking waves in single wave gauge records of surface elevation fluctuations.
% J. Atmos. Ocean. Technol. JTECH-D-19-0011.1.
% https://doi.org/10.1175/JTECH-D-19-0011.1

% Please cite JTECH-D-19-0011.1 and JPO-D-16-0289.1

% Input arguments:
% FileName - is the surface elevation fluctuations data file. The data is expected to be in ASCII fromat,
%   with the surface elevation fluctuations time series as columns. Multiple wave gauges data are allowed,
%   each gauge data constituting a column. In this case the function will loop on gauges data,
%   generating combined statistics.
% Fs - is the data sampling frequency [Hz]
% NumGauges - is the number of gauges data (columns) to process. The first
%   NumGuges columns will be processed
% plots - is teh binary flag [0/1], turbning plotting of the results on/off
% Fs is teh sampling frequency,
%
% Output arguments:
% Nb - is the count of total detected breaking crests
% Nnb - is the count of total detected non-breaking crests
% Ab - is the vector containing values of the detected breaking crests amplitudes (units simmilar to the input)
% Anb - is the vector containing values of the detected non-breaking crests amplitudes (units simmilar to the input)
% Dfb - is the vector containing values of the detected breaking crests (wave) frequencies [Hz]
% Dfnb - is the vector containing values of the detected non-breaking crests (wave) frequencies [Hz]
%
% Dan Liberzon, 2019, liberzon@technion.ac.il
%% Basic Settings
FN='F3.mat'; % Wavelet to use, by name, must be in the same Folder
DetPRCT=0.6; % prct (%) of max energy (%Emax) - a threshold for detection
Fc=30;       % Low Pass cutoff frequency [Hz] for surface leevtaion  and F(t) data, 0 if not needed
fc=10;       % Low Pass cutoff frequency for Wavelets Coefficients filtering [Hz]
Noct=64;     % N. of octaves for wavelets resolution

%% Load and prepare data: substructing averages, filtering, etc.

data = load(FileName);
[n,m]=size(data);
if m>n; data=data';end; % make sure data is in columns
Eta=zeros(length(data),NumGauges);
for i=1:NumGauges
    Eta(:,i)=data(:,i)-mean(data(:,i));
end
clear data;

%%
% Loop on WG channels and perform all calculations
Ab=[];Anb=[];Dfb=[];Dfnb=[];
for et=1:NumGauges
    eta=Eta(:,et);
    % Obtain frr F(s) by Hilbert transform:
    [t,frr]=FreqVarHilbertClean(eta, Fs);
    % Condition the signal (filtering, amplification, crests detection)
    [fr, tmax, maxe, t_pos, t_neg]=SigCondit(eta, frr, Fs, Fc);
    
    
    
    % Plot only the first channel to get the general idea of quality:
    if plots & et==1
        figure(1);
        
        a1=subplot(2,1,1);% plot eta and crests
        plot(t,eta,'-b'); grid on;
        hold on; scatter(tmax,maxe,20,'ro','filled');
        xlabel('t [s]');ylabel('\eta [mm]');
        legend('\eta','crests');
        
        a2=subplot(2,1,2);% plot inst. frequency and filtered inst. frequency
        plot(t,frr,'k');
        hold on;
        plot(t,fr,'r');
        grid on;
        % ylim([-100 100]);
        xlabel('t [s]');ylabel('F(t) [mm\cdotrad/sec]');
        legend('F(t)','F(t)- low-pass filtered and amplified');
        hold off;
        linkaxes([a1,a2],'x');
        
    end
    
    % Do Wavelets analyses to detect the breaking associated patterns:
    % Wavelets:
    [C, fsc]=PatternRecWavelet(fr, Fs, FN, Noct);
    
    % Low-Pass Filter Coefficients for smoothing
    Csl=LowPass2D(C,Fs,fc);
    % Detect Breakers by threshold (DetPRCT)
    [Ndet, Ndetprct, Is, Cc]=DetectBreaksLoc(Csl, DetPRCT, Fs, tmax);
    
    % Plot again, this time the detected breakers positions, coefs. maps, etc.
    if plots & et==1
        figure(2)
        
        a21=subplot(3,1,1);% plot eta and peaks
        plot(t,eta,'-b'); grid on;
        hold on; scatter(tmax,maxe,20,'ro','filled');
        plot(t(round(Is)),zeros(length(Is),1),'kx','MarkerSize',10,'LineWidth',1);
        xlabel('t [s]');ylabel('\eta [mm]');
        legend('\eta','crests', 'breakers');
        
        a22=subplot(3,1,2);% plot frequency and filtered frequency
        plot(t,fr,'k'); grid on
        xlabel('t [s]');ylabel('F [mm \cdot rad \cdot Hz]');
        legend('F(t) low-pass filtered and amplified');
        hold off;
        
        a23=subplot(3,1,3);% plot instantanious frequency
        imagesc(t,fsc,Csl);
        xlabel('t [s]');
        ylabel ('\omega_{F} [Hz]');
        
        linkaxes([a21,a22,a23],'x');
        
        
    end
    
    % Collect statistics for the currently being processed WG channel
    % Use t_pos and t_neg to find breaking and non-breaking crests amplitudes and wave frequency
    % Start and run on all tmax values
    ab=[]; anb=[]; dfb=[]; dfnb=[];
    
    dtpos=tmax-t_pos(1:end-1);
    dtpose=t_pos(2:end)-tmax;
    df=1./(dtpos+dtpose);% wave frequency between two t_pos
    df(df>10)=NaN; % anomalously short waves are not accounted for
    tb=t(round(Is(~isnan(Is))));
    for i=1:length(tmax)
        if min(abs(tb-tmax(i)))<(1/(2*df(i))) % breaking crest
            ab=[ab; maxe(i)];
            dfb=[dfb; df(i)];
        else
            anb=[anb; maxe(i)];
            dfnb=[dfnb; df(i)];
        end
        
    end
    % end looping on the current WG channels, start consolidating the results for the output
    Ab=[Ab;ab];
    Anb=[Anb;anb];
    Dfb=[Dfb;dfb];
    Dfnb=[Dfnb;dfnb];
end
Nb=length(Ab); Nnb=length(Anb);



